;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : 6-stage phaser
;
; Description : Phaser with 6 dependent stages
;
; Nb In       : 1
; Nb Out      : 1
;
; Input       : Processed signal
; Output      : ?
;
; Cycles      : 119 + 50 cycles
; Memory      : 32 words (without init)
; Delay Line  : no
;
; Author      : AgALag (Axoris application generator)
; Date        : 
; Version     : 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; INIT memory address
ABS INIT 0x3FF

	;; SEC1 Control parameters
ABS	constant1Cst	0x0
ABS	triangle6Ampl	0x1
ABS	triangle6Delta	0x2
ABS	Mixer2to12Scale1	0x5
ABS	Mixer2to12Scale2	0x6
ABS	Feedbackpart19Feedback	0x8
ABS	tri2sinappx55Ampl	0x9
ABS	InterpolateExp11dMin	0xB
ABS	InterpolateExp11dMax	0xC
ABS	InterpolateLin10dMin	0xD
ABS	InterpolateLin10dMax	0xE
ABS	Mixer2to17Scale1	0xF
ABS	Mixer2to17Scale2	0x10
ABS	gain4Gain	0x18
ABS	Mixer2to118Scale1	0x1A
ABS	Mixer2to118Scale2	0x1B
	;; SEC2 Initial values of control parameters (S3.24 format)
constant1ICst EQU 0x1000000
	
triangle6IAmpl EQU 1.0
triangle6IDelta EQU 0x00002bb
; 0x00a377f for 440 Hz
	
Mixer2to12IScale1 EQU 1.0
Mixer2to12IScale2 EQU 0.0

Feedbackpart19IFeedback EQU 0.5

tri2sinappx55IAmpl EQU 1.0
	
InterpolateExp11IdMin EQU 0.02
InterpolateExp11IdMax EQU 0.1

InterpolateLin10IdMin EQU 0.02
InterpolateLin10IdMax EQU 0.1

Mixer2to17IScale1 EQU 0.0
Mixer2to17IScale2 EQU 1.0


;none

;none

;none

;none

;none

;none

gain4IGain EQU 1.0
	
;none

Mixer2to118IScale1 EQU 0.5
Mixer2to118IScale2 EQU 0.5

	;; SEC3 Constants definitions


;none

;none

;; These are the coefficients of the polynome transforming the triangle into sine wave.
;; The coefficients are used in instructions using S3.18 format.
tri2sinappx55Coeffx1 EQU 0x06487E ; = pi/2
tri2sinappx55Coeffx3 EQU 0x3D6F03 ; = (5/2 - pi)
tri2sinappx55Coeffx5 EQU 0x00487E ; = (pi/2 - 3/2)



;none


;none

;none

;none

;none

;none

;none


;none

;none

	;; SEC4 Memory variables definition
ABS	triangle6Up	0x3
ABS	triangle6Value	0x4
ABS	Feedbackpart19FbState	0x7
ABS	tri2sinappx55Tmp	0xA
ABS	PhaserCoeffCalc12TempValue	0x11
ABS	AllPassFilter17zm1	0x12
ABS	AllPassFilter16zm1	0x13
ABS	AllPassFilter15zm1	0x14
ABS	AllPassFilter14zm1	0x15
ABS	AllPassFilter13zm1	0x16
ABS	AllPassFilter3zm1	0x17
ABS	Feedbackpart28FbState	0x7
;; temporary IO for modules intercommunications
ABS	TMP0	0x1C
ABS	TMP1	0x1D
ABS	TMP2	0x1E
ABS	TMP3	0x1F
	;; SEC5 Initial values of memory variables

triangle6IUp EQU 0x1000000
triangle6IValue EQU 0x0
	
;none

Feedbackpart19IFbState EQU 0.0

	


;none


;none

;none

;none

;none

;none

;none


Feedbackpart28IFbState EQU 0.0

;none

	;; SEC6 Initialization
CM      0x40000 INIT
SKIP    !Z      EndInit
C constant1ICst
SCA 0x0 constant1Cst

C triangle6IValue ; load triangle6Value
SCA 0x0 triangle6Value
C triangle6IUp ; load triangle6Value
SCA 0x0 triangle6Up
C triangle6IAmpl ; load triangle6Value
SCA 0x0 triangle6Ampl
C triangle6IDelta ; load triangle6Value
SCA 0x0 triangle6Delta

C Mixer2to12IScale1 ; load Mixer2to12IScale1
SCA 0x0 Mixer2to12Scale1 ; save to Mixer2to12Scale1
C Mixer2to12IScale2 ; load Mixer2to12IScale2
SCA 0x0 Mixer2to12Scale2 ; save to Mixer2to12Scale2

C Feedbackpart19IFeedback ; load Feedbackpart19IFeedback
SCA 0x0 Feedbackpart19Feedback ; save to Feedbackpart19Feedback
C Feedbackpart19IFbState ; load Feedbackpart19IFbState
SCA 0x0 Feedbackpart19FbState ; save to Feedbackpart19FbState

C tri2sinappx55IAmpl ; load value
SCA 0.0 tri2sinappx55Ampl

C InterpolateExp11IdMin ; load init value for InterpolateExp11IdMin ...
SCA 0x0 InterpolateExp11dMin ; ... Stored in InterpolateExp11dMin
C InterpolateExp11IdMax ; load init value for InterpolateExp11IdMax ...
SCA 0x0 InterpolateExp11dMax ; ... Stored in InterpolateExp11dMax

C InterpolateLin10IdMin ; load init value for InterpolateLin10IdMin ...
SCA 0x0 InterpolateLin10dMin ; ... Stored in InterpolateLin10dMin
C InterpolateLin10IdMax ; load init value for InterpolateLin10IdMax ...
SCA 0x0 InterpolateLin10dMax ; ... Stored in InterpolateLin10dMax

C Mixer2to17IScale1 ; load Mixer2to17IScale1
SCA 0x0 Mixer2to17Scale1 ; save to Mixer2to17Scale1
C Mixer2to17IScale2 ; load Mixer2to17IScale2
SCA 0x0 Mixer2to17Scale2 ; save to Mixer2to17Scale2

SCA 0x0 AllPassFilter17zm1 ; initialize state to 0

SCA 0x0 AllPassFilter16zm1 ; initialize state to 0

SCA 0x0 AllPassFilter15zm1 ; initialize state to 0

SCA 0x0 AllPassFilter14zm1 ; initialize state to 0

SCA 0x0 AllPassFilter13zm1 ; initialize state to 0

SCA 0x0 AllPassFilter3zm1 ; initialize state to 0

1AC gain4IGain ; initial default gain4Gain value
SCA 0x0 gain4Gain

C Feedbackpart28IFbState ; load Feedbackpart28IFbState
SCA 0x0 Feedbackpart28FbState ; save to Feedbackpart28FbState

C Mixer2to118IScale1 ; load Mixer2to118IScale1
SCA 0x0 Mixer2to118Scale1 ; save to Mixer2to118Scale1
C Mixer2to118IScale2 ; load Mixer2to118IScale2
SCA 0x0 Mixer2to118Scale2 ; save to Mixer2to118Scale2

C 0x1000000	; Set A to 1
SCA 0x0 INIT	; do not reinitialise next time

EndInit:

	;; SEC7 Module code
CM 0x40000 constant1Cst
SCA 0x0 TMP0
CM 0x40000 triangle6Value ; triangle6Value -> A
XCM 0x40000 triangle6Delta ; triangle6Value -> B, triangle6Delta -> A
AMB triangle6Up ; triangle6Value +/- triangle6Delta -> A
AMC 0x3c0000 triangle6Up ; |'new'-Value| - 1 -> A
SKIP N+Z triangle6Output
;; we have |overflow| in A
;; we must 1) invert triangle6Up
;; 2) put |overflow|-2 in A
XCM 0x3c0000 triangle6Up ; save |overflow| in B, load -up
S1BC 0x380000 triangle6Up ; Save new triangle6Up value, |overflow|-2 -> A
triangle6Output:
DAC 0x100 0x40000 ; A+1
AMC 0x0 triangle6Up ; New A calculated
SCA 0x40000 triangle6Value ; update triangle6Value
AMC 0x0 triangle6Ampl ; rescale
SCA 0x0 TMP1 ; write triangle6Output
;;; Mixing the inputs
CM 1.0 Mixer2to12Scale1 ; A = Mixer2to12Scale1
AMC 0.0 TMP1 ; A = Mixer2to12Scale1 * input1
XCM 1.0 Mixer2to12Scale2 ; A = Mixer2to12Scale2 B = Mixer2to12Scale1 * input1
AMB TMP0 ; A = Mixer2to12Scale2 * input2 + Mixer2to12Scale1 * input1
SCA 0.0 TMP2 ; output = A
;;; Getting the Feedbackpart19Feedback and adding it to the input
1MC 0.0 Feedbackpart19FbState ; A = Feedbackpart19FbState
AMC 0.0 Feedbackpart19Feedback ; A = Feedbackpart19FbState * Feedbackpart19Feedback
CMA 1.0 IN1 ; A = InputSample + Feedbackpart19FbState * Feedbackpart19Feedback
SCA 0.0 TMP0 ; TMP0 = InputSample + Feedbackpart19FbState * Feedbackpart19Feedback

CM 1.0 TMP2 ; Value -> A
; transform now the triangle into a sine by applying the polynome
SXAAC 0.0 tri2sinappx55Tmp ; dummy update tri2sinappx55Tmp
; A = x^2 B = x
AAC 0.0 ; A = x^4
BAC 0.0 ; A = x^5
CAD tri2sinappx55Coeffx5 0.0 ; A = 0.25 * x^5
SBBC 0.0 tri2sinappx55Tmp ; A = x^2 tri2sinappx55Tmp = 0.25 * x^5
BAC 0.0 ; A = x^3
CAM tri2sinappx55Coeffx3 tri2sinappx55Tmp ; A = -0.5 * x^3 + 0.25 * x^5
SCBA tri2sinappx55Coeffx1 tri2sinappx55Tmp ; A = 1.5 * x - 0.5 * x^3 + 0.25 * x^5
AMC 0.0 tri2sinappx55Ampl ; rescale
SCA 1.0 TMP1 ; write output

;;; Interpolate the value
1MC 1.0 TMP2 ; A = (triangle(n) + 1.0)
CAD 0.25 0.0 ; A = (triangle(n) + 1.0) / 4
X1AC 0.0 ; B = (triangle(n) + 1.0) / 4
EXPB ; A = exp16(B)
1AC -1.0 ; A = exp16 - 1.0
XCM 0.3334 InterpolateExp11dMax ; B = exp16 - 1.0 A = InterpolateExp11dMax / 3.0
CMA -0.3334 InterpolateExp11dMin ; A = (dMax - dMin) / 3.0
BAC 0.0 ; A = (dMax - dMin) * ratio / 3.0
CAM 1.0 InterpolateExp11dMin ; A = InterpolateExp11dMin + (dMax - dMin) * ratio / 3.0
SCA 1.0 TMP3 ; TMP3 = InterpolateExp11dMin + (dMax - dMin) * ratio where ratio = [0.0 ... 1.0]

;;; Interpolate the value
CM 0.5 InterpolateLin10dMax ; A = InterpolateLin10dMax / 2.0
CMA -0.5 InterpolateLin10dMin ; A = (dMax - dMin) / 2.0
AMC 0.0 TMP1 ; A = TMP1 * (dMax - dMin) / 2.0
CMA 0.5 InterpolateLin10dMin ; A = dMin/2 + TMP1 * (dMax - dMin) / 2.0
CMA 0.5 InterpolateLin10dMax ; A = dMax/2 - dMin/2 + TMP1 * (dMax - dMin) / 2.0
SCA 1.0 TMP2 ; TMP2 = InterpolateLin10dMin + TMP1 * (dMax - dMin) / 2.0 with TMP1 in [-1.0 ... 1.0]
; TMP2 = InterpolateLin10dMin + (dMax - dMin) * ratio with ratio in [0.0 ... 1.0]

;; InterpolateLin10dMin + (dMax - dMin) * ratio where ratio is in [0.0 ... 1.0]
;; = InterpolateLin10dMin + (dMax - dMin) * (IN1 + 1.0) / 2.0 where TMP1 is in [-1.0 ... 1.0]
;; = InterpolateLin10dMin + (dMax - dMin) / 2.0 + TMP1 * (dMax - dMin) / 2.0
;; = dMin/2 + dMax/2 + TMP1 * (dMax - dMin)/2
;;; Mixing the inputs
CM 1.0 Mixer2to17Scale1 ; A = Mixer2to17Scale1
AMC 0.0 TMP2 ; A = Mixer2to17Scale1 * input1
XCM 1.0 Mixer2to17Scale2 ; A = Mixer2to17Scale2 B = Mixer2to17Scale1 * input1
AMB TMP3 ; A = Mixer2to17Scale2 * input2 + Mixer2to17Scale1 * input1
SCA 0.0 TMP1 ; output = A

;;; Generating then the all-pass coefficients from that
1MC 1.0 TMP1 ; A = 1 + x
XCM 0.0 PhaserCoeffCalc12TempValue ; A -> B A = 0 * PhaserCoeffCalc12TempValue
LOGB ; A = log16(1 + x)
LCA -1.0 PhaserCoeffCalc12TempValue ; A = -log16(1 + x) B = dummy
XCM 0.0 PhaserCoeffCalc12TempValue ; A -> B A = 0 * PhaserCoeffCalc12TempValue
EXPB ; A = ~ 1 / (1 + x)
SCA 0.0 PhaserCoeffCalc12TempValue ; mem[TempValue] = ~ 1 / (1 + x)
LCA 0.0 TMP1 ; B = x (= mem[coeff])
CB -1.0 ; A = -x
1AC 1.0 ; A = 1 - x
AMC 0.0 PhaserCoeffCalc12TempValue ; A = (1 - x) * (~ 1 / (1 + x))
SCA 0.0 TMP2 ; mem[coeff] = A

;;; Applying the all-pass
CM -1.0 TMP2 ; A = -coeff
AMC 0.0 TMP0 ; A = InputValue * -coeff
CAM 1.0 AllPassFilter17zm1 ; A = InputValue * -coeff + AllPassFilter17zm1
SCA 1.0 TMP1 ; TempValue = y = A, don't touch A it will be re-used

AMC 0.0 TMP2 ; A = y * coeff
CAM 1.0 TMP0 ; A = y * coeff + InputSample
SCA 1.0 AllPassFilter17zm1 ; zm11 = A


;;; Applying the all-pass
CM -1.0 TMP2 ; A = -coeff
AMC 0.0 TMP1 ; A = InputValue * -coeff
CAM 1.0 AllPassFilter16zm1 ; A = InputValue * -coeff + AllPassFilter16zm1
SCA 1.0 TMP0 ; TempValue = y = A, don't touch A it will be re-used

AMC 0.0 TMP2 ; A = y * coeff
CAM 1.0 TMP1 ; A = y * coeff + InputSample
SCA 1.0 AllPassFilter16zm1 ; zm11 = A


;;; Applying the all-pass
CM -1.0 TMP2 ; A = -coeff
AMC 0.0 TMP0 ; A = InputValue * -coeff
CAM 1.0 AllPassFilter15zm1 ; A = InputValue * -coeff + AllPassFilter15zm1
SCA 1.0 TMP1 ; TempValue = y = A, don't touch A it will be re-used

AMC 0.0 TMP2 ; A = y * coeff
CAM 1.0 TMP0 ; A = y * coeff + InputSample
SCA 1.0 AllPassFilter15zm1 ; zm11 = A


;;; Applying the all-pass
CM -1.0 TMP2 ; A = -coeff
AMC 0.0 TMP1 ; A = InputValue * -coeff
CAM 1.0 AllPassFilter14zm1 ; A = InputValue * -coeff + AllPassFilter14zm1
SCA 1.0 TMP0 ; TempValue = y = A, don't touch A it will be re-used

AMC 0.0 TMP2 ; A = y * coeff
CAM 1.0 TMP1 ; A = y * coeff + InputSample
SCA 1.0 AllPassFilter14zm1 ; zm11 = A


;;; Applying the all-pass
CM -1.0 TMP2 ; A = -coeff
AMC 0.0 TMP0 ; A = InputValue * -coeff
CAM 1.0 AllPassFilter13zm1 ; A = InputValue * -coeff + AllPassFilter13zm1
SCA 1.0 TMP1 ; TempValue = y = A, don't touch A it will be re-used

AMC 0.0 TMP2 ; A = y * coeff
CAM 1.0 TMP0 ; A = y * coeff + InputSample
SCA 1.0 AllPassFilter13zm1 ; zm11 = A


;;; Applying the all-pass
CM -1.0 TMP2 ; A = -coeff
AMC 0.0 TMP1 ; A = InputValue * -coeff
CAM 1.0 AllPassFilter3zm1 ; A = InputValue * -coeff + AllPassFilter3zm1
SCA 1.0 TMP0 ; TempValue = y = A, don't touch A it will be re-used

AMC 0.0 TMP2 ; A = y * coeff
CAM 1.0 TMP1 ; A = y * coeff + InputSample
SCA 1.0 AllPassFilter3zm1 ; zm11 = A


CM 0x40000 TMP0 ; load input
AMC 0x0 gain4Gain ; amplify
SCA 0x0 TMP1 ; write output and restore 0 in A
;;; Getting the feedback and adding it to the input
1MC 0.0 TMP0 ; A = TMP0
SCA 1.0 Feedbackpart28FbState ; Update the feedback path with the new input

;;; Mixing the inputs
CM 1.0 Mixer2to118Scale1 ; A = Mixer2to118Scale1
AMC 0.0 IN1 ; A = Mixer2to118Scale1 * input1
XCM 1.0 Mixer2to118Scale2 ; A = Mixer2to118Scale2 B = Mixer2to118Scale1 * input1
AMB TMP1 ; A = Mixer2to118Scale2 * input2 + Mixer2to118Scale1 * input1
SCA 0.0 OUT1 ; output = A
